import sys
import os
import Numeric
import rpy



config_dic = {
    "source_dir" : os.path.join(os.environ['STRUSTER'],'SRC'),
    "log_file"   : 'job_bck_comp.rpt',
    "sets_file"  : os.path.join(os.environ['STRUSTER'],'DATA/SETS/51459_dir.cla.scop_1.71_subset.txt'),
    "pdb_dir"    : os.path.join(os.environ['STRUSTER'],'DATA/PDB'),
    "tmp_dir"    : os.path.join(os.environ['STRUSTER'],'RESULTS/TMP'),
    "pdb_superp_dir": os.path.join(os.environ['STRUSTER'],'DATA/PDB'),
    "use_pdb_subdir" :  False,
    "pdb_superp_type" : 'astral',
    "stmap_dir"  : None,
    "uniprot_dir" : os.path.join(os.environ['STRUSTER'],'DATA/UNIPROT'),
    "dpi_file"   : os.path.join(os.environ['STRUSTER'],'DATA/SUS/example_dpi.sus'),
    "min_set_len" : 2,
    "max_set_len" : 60,    
    "dist_filter" : 14,
    "dif_dist_filter" : 1,
    "one_gap_value" : 0.1,
    "two_gap_value": 0.0,
    "get_superp" : True,
    "method"     : 'sus',
    "cl_atoms" :   'CA',    
    'atom_sel'   : 'CA'
    }


# import struster modules
sys.path.append(config_dic['source_dir'])

import log_util
import clustering
import compare_sets
import set_alig
import  scop_sets
import hinge_inv
import util_r
import superposition


# start logging to the log_file
log_util.log_to_file(config_dic['log_file'])

util_r.source_r_plots(config_dic['source_dir'])
util_r.source_r_clustering(config_dic['source_dir'])

# define subsets
subset_d1dhka2_lst = ['d1dhka2']
subset_d1bvnp2_lst = ['d1bvnp2']
subset_A_lst = ['d1kxva2', 'd1kxqa2', 'd1kxqc2','d1kxqb2','d1kxqd2','d1kxte2','d1kxtc2','d1kxvb2','d1kxta2','d1ua3a2','d1pif_2']
subset_B_lst = ['d1jfh_2', 'd1hx0a2', 'd1pig_2', 'd1ose_2']
subset_AB_lst = subset_A_lst + subset_B_lst
subset_1dhk_1bvn_A_lst = subset_d1dhka2_lst + subset_d1bvnp2_lst + subset_A_lst
set_lst = subset_d1dhka2_lst + subset_d1bvnp2_lst +subset_AB_lst



# get baseset
read_scop = scop_sets.ReadScopBaseset(config_dic['min_set_len'], config_dic['max_set_len'])
read_scop.load_file(config_dic['sets_file'])
baseset = read_scop.get_baseset(config_dic['pdb_dir'], config_dic['use_pdb_subdir'])
baseset.set_working_to_current_dir()
# set_lst = baseset.get_sentry_id_lst()
    
# get aligned distance array
alig_file = os.path.join(os.environ['STRUSTER'],'DATA/SETS/51459_alig.fasta')
baseset.get_aligment_from_msa(alig_file)
baseset.get_alig_dist_array()
baseset.get_alig_sus(dpi_file = config_dic['dpi_file'])
    
# comparison
compsets = compare_sets.CompSets(baseset, config_dic['source_dir'])
compsets.comparison('51459_comp_A:B_U_5', subset_A_lst, subset_B_lst, 'sus', 'dscore', 5.0, 'CA')
hi_obj = hinge_inv.HingeInv('51459_comp_A:B_U_5:0.1:0.2', config_dic['source_dir'], '.', True)
hi_obj.get_hinges_invariant(score_mat = compsets.score_mat, undef_mat = compsets.undef_mat, score_cutoff = 5.0, filter_cutoff = 0.1, min_gap = 10, inv_cutoff=0.2, mat_type= 'comp')
    

# finish logging
log_util.close_log()
