import sys
import os
import Numeric
import rpy



config_dic = {
    "source_dir" : os.path.join(os.environ['STRUSTER'],'SRC'),
    "log_file"   : 'job_bck_var.rpt',
    "sets_file"  : os.path.join(os.environ['STRUSTER'],'DATA/SETS/51459_dir.cla.scop_1.71_subset.txt'),
    "pdb_dir"    : os.path.join(os.environ['STRUSTER'],'DATA/PDB'),
    "tmp_dir"    : os.path.join(os.environ['STRUSTER'],'RESULTS/TMP'),
    "pdb_superp_dir": os.path.join(os.environ['STRUSTER'],'DATA/PDB'),
    "use_pdb_subdir" :  False,
    "pdb_superp_type" : 'astral',
    "stmap_dir"  : None,
    "uniprot_dir" : os.path.join(os.environ['STRUSTER'],'DATA/UNIPROT'),
    "dpi_file"   : os.path.join(os.environ['STRUSTER'],'DATA/SUS/example_dpi.sus'),
    "min_set_len" : 2,
    "max_set_len" : 60,    
    "dist_filter" : 14,
    "dif_dist_filter" : 1,
    "one_gap_value" : 0.1,
    "two_gap_value": 0.0,
    "get_superp" : True,
    "method"     : 'sus',
    "cl_atoms" :   'CA',    
    'atom_sel'   : 'CA'
    }


# import struster modules
sys.path.append(config_dic['source_dir'])

import log_util
import clustering
import compare_sets
import set_alig
import  scop_sets
import hinge_inv
import util_r
import superposition


# start logging to the log_file
log_util.log_to_file(config_dic['log_file'])

util_r.source_r_plots(config_dic['source_dir'])
util_r.source_r_clustering(config_dic['source_dir'])






# get baseset
read_scop = scop_sets.ReadScopBaseset(config_dic['min_set_len'], config_dic['max_set_len'])
read_scop.load_file(config_dic['sets_file'])
baseset = read_scop.get_baseset(config_dic['pdb_dir'], config_dic['use_pdb_subdir'])
baseset.set_working_to_current_dir()
set_lst = baseset.get_sentry_id_lst()

# write sequences as fasta file
uniprot_id_lst = ['P00690']
fasta_file = "example_set.fasta"
baseset.write_sequences_fasta(config_dic['uniprot_dir'], uniprot_id_lst, fasta_file)

    
# get aligned distance array
alig_file = os.path.join(os.environ['STRUSTER'],'DATA/SETS/51459_alig.fasta')
# Use Aligned sequences
baseset.get_aligment_from_msa(alig_file)
# Use PDB/UNIPROT mapping from PDBSWS
# stmap_dir = '/home/doming/WORK/STRUSTER/V_3_1/STMAP_DIR'
# uniprot_dir = '/home/doming/WORK/STRUSTER/V_3_1/UNIPROT_PDBSWS/SEQ'
# baseset.get_alignment(stmap_dir,uniprot_dir)
baseset.get_alig_dist_array()
baseset.get_alig_sus(dpi_file = config_dic['dpi_file'])
    
# write alignment information
baseset.alig.write_alignment('51459.alig')
# baseset.alig.write_fasta_alig('51459_alig.fasta')
# sus boxplots
baseset.sus_boxplots_subset(set_lst, '51459_set_sus_boxplots', config_dic['source_dir'])

# clustering
clusterset =  clustering.ClusterSet(baseset, config_dic['dist_filter'], config_dic['dif_dist_filter'], config_dic['one_gap_value'], config_dic['two_gap_value'], config_dic['get_superp'], config_dic['cl_atoms'])
# clusterset.baseset.get_alig_dist_array()
clusterset.cluster_dmat()


# variation
# S
s_mat = baseset.get_dist_sdev_mat(set_lst, 'CA')
baseset.plot_sdev(s_mat, '51459_set_S')
    
# R
nmat = baseset.get_sqrtmean_dsusv_mat(set_lst, 'CA')
r_mat = baseset.get_rel_dist_sdev_mat(s_mat, nmat)
baseset.plot_relsdev(r_mat, '51459_set_R_mean')

# T
t_mat = baseset.get_dist_sus_sdev_mat(set_lst, s_mat, 'CA')
baseset.plot_sdev(t_mat, '51459_set_T')        
# detect hinges invariant regions
und_mat = baseset.get_undef_mat(t_mat)    
hi_obj = hinge_inv.HingeInv('51459_set_T_1:0.1:0.2', config_dic['source_dir'], '.', True)
hi_obj.get_hinges_invariant(score_mat = t_mat, undef_mat = und_mat, score_cutoff = 1.0, filter_cutoff = 0.1, min_gap = 10, inv_cutoff =0.2, mat_type= 'variation')    
# superposition based on T
max_superp = 3
superp_counter = 0
for inv_region in hi_obj.inv_region_lst:
    if superp_counter > max_superp:
        break
    superp = superposition.Superp(config_dic, baseset, set_lst, clusterset.fdif_dic)
    superp_pdb_filename = baseset.set_id + '_superp_T_1:0.1:0.2_'+ str(superp_counter+1) + '.pdb'
    superp_pdb_file = os.path.join(baseset.working_dir, superp_pdb_filename)
    superp.superimpose_subsets(inv_region, superp_pdb_file)
    datafilename = baseset.set_id + '_superp_T_1:0.1:0.2_'+ str(superp_counter+1) + '.data'
    datafile = os.path.join(baseset.working_dir, datafilename)
    superp.save_superp_data(datafile)
    superp_rasm_filename = baseset.set_id + '_superp_T_1:0.1:0.2_'+ str(superp_counter+1) + '.rsm'
    superp_rasm_file = os.path.join(baseset.working_dir, superp_rasm_filename)
    superp.get_rasmol_script(superp_pdb_file, inv_region, hi_obj.hinges_lst, superp_rasm_file)
    superp_counter += 1



# X
x_mat = baseset.get_x_mat(set_lst, 'CA')
baseset.plot_variation_X(x_mat, '51459_set_X')
    

# finish logging
log_util.close_log()
