import sys
import os
import Numeric
import rpy



config_dic = {
    "source_dir" : os.path.join(os.environ['STRUSTER'],'SRC'),
    "log_file"   : 'job_sc_comp.rpt',
    "sets_file"  : os.path.join(os.environ['STRUSTER'],'DATA/SETS/51459_dir.cla.scop_1.71_subsetAB.txt'),
    "pdb_dir"    : os.path.join(os.environ['STRUSTER'],'DATA/PDB'),
    "tmp_dir"    : os.path.join(os.environ['STRUSTER'],'RESULTS/TMP'),
    "pdb_superp_dir": os.path.join(os.environ['STRUSTER'],'DATA/PDB'),
    "use_pdb_subdir" :  False,
    "pdb_superp_type" : 'astral',
    "stmap_dir"  : None,
    "uniprot_dir" : os.path.join(os.environ['STRUSTER'],'DATA/UNIPROT'),
    "dpi_file"   : os.path.join(os.environ['STRUSTER'],'DATA/SUS/example_dpi.sus'),
    "min_set_len" : 2,
    "max_set_len" : 60,    
    "dist_filter" : 14,
    "dif_dist_filter" : 0.5,
    "one_gap_value" : 0.1,
    "two_gap_value": 0.0,
    "get_superp" : True,
    "method"     : 'sus',
    "cl_atoms" :   'CA_GC',    
    'atom_sel'   : 'GC'
    }


# import struster modules
sys.path.append(config_dic['source_dir'])

import log_util
import clustering
import compare_sets
import set_alig
import  scop_sets
import hinge_inv
import util_r
import superposition


# start logging to the log_file
log_util.log_to_file(config_dic['log_file'])

util_r.source_r_plots(config_dic['source_dir'])
util_r.source_r_clustering(config_dic['source_dir'])

# define subsets
subset_E1_lst = ['d1kxqa2', 'd1kxqc2','d1kxqb2','d1kxqd2']
subset_E2_lst = ['d1kxva2', 'd1kxte2','d1kxtc2','d1kxvb2','d1kxta2','d1ua3a2','d1pif_2']
subset_E_lst = ['d1kxva2', 'd1kxqa2', 'd1kxqc2','d1kxqb2','d1kxqd2','d1kxte2','d1kxtc2','d1kxvb2','d1kxta2','d1ua3a2','d1pif_2']
subset_B_lst = ['d1jfh_2', 'd1hx0a2', 'd1pig_2', 'd1ose_2']
subset_EB_lst = subset_E_lst + subset_B_lst


# get baseset
read_scop = scop_sets.ReadScopBaseset(config_dic['min_set_len'], config_dic['max_set_len'], config_dic['atom_sel'])
read_scop.load_file(config_dic['sets_file'])
baseset = read_scop.get_baseset(config_dic['pdb_dir'], config_dic['use_pdb_subdir'])
baseset.set_working_to_current_dir()
set_lst = baseset.get_sentry_id_lst()

# get aligned distance array
alig_file = os.path.join(os.environ['STRUSTER'],'DATA/SETS/51459_subsetAB_alig.fasta')
baseset.get_aligment_from_msa(alig_file)
baseset.get_alig_dist_array()
baseset.get_alig_gc_dist_array()
baseset.get_alig_sus(dpi_file = config_dic['dpi_file'])
baseset.get_alig_gc_sus(dpi_file = config_dic['dpi_file'])

# define ligand binding site, from d1pig_2
site_sentry_id = 'd1pig_2'
site_reskey_lst = [' :58 ', ' :59 ', ' :60 ', ' :61 ', ' :62 ', ' :63 ', ' :98 ', ' :101 ', ' :103 ', ' :104 ', ' :105 ', ' :151 ', ' :162 ', ' :163 ', ' :164 ',' :165 ', ' :195 ', ' :197 ', ' :198 ', ' :199 ', ' :200 ', ' :201 ', ' :233 ', ' :235 ', ' :236 ', ' :237 ', ' :238 ', ' :240 ', ' :298 ', ' :299 ', ' :300 ', ' :305 ', ' :306 ', ' :307 ', ' :308 ']

# comparison
compsets = compare_sets.CompSets(baseset, config_dic['source_dir'])

compsets.comparison('51459_comp_E:B_U_5_GC', subset_E_lst, subset_B_lst, 'sus', 'dscore', 5.0, 'GC')
compsets.plot_score_submat(site_sentry_id, site_reskey_lst)

compsets.comparison('51459_comp_E1:E2_U_5_GC', subset_E1_lst, subset_E2_lst, 'sus', 'dscore', 5.0, 'GC')
compsets.plot_score_submat(site_sentry_id, site_reskey_lst)



# finish logging
log_util.close_log()
