import sys
import os
import Numeric
import rpy



config_dic = {
    "source_dir" : os.path.join(os.environ['STRUSTER'],'SRC'),
    "log_file"   : 'job_sc_var.rpt',
    "sets_file"  : os.path.join(os.environ['STRUSTER'],'DATA/SETS/51459_dir.cla.scop_1.71_subsetAB.txt'),
    "pdb_dir"    : os.path.join(os.environ['STRUSTER'],'DATA/PDB'),
    "tmp_dir"    : os.path.join(os.environ['STRUSTER'],'RESULTS/TMP'),
    "pdb_superp_dir": os.path.join(os.environ['STRUSTER'],'DATA/PDB'),
    "use_pdb_subdir" :  False,
    "pdb_superp_type" : 'astral',
    "stmap_dir"  : None,
    "uniprot_dir" : os.path.join(os.environ['STRUSTER'],'DATA/UNIPROT'),
    "dpi_file"   : os.path.join(os.environ['STRUSTER'],'DATA/SUS/example_dpi.sus'),
    "min_set_len" : 2,
    "max_set_len" : 60,    
    "dist_filter" : 14,
    "dif_dist_filter" : 0.5,
    "one_gap_value" : 0.1,
    "two_gap_value": 0.0,
    "get_superp" : True,
    "method"     : 'sus',
    "cl_atoms" :   'CA_GC',    
    'atom_sel'   : 'GC'
    }


# import struster modules
sys.path.append(config_dic['source_dir'])

import log_util
import clustering
import compare_sets
import set_alig
import  scop_sets
import hinge_inv
import util_r
import superposition


# start logging to the log_file
log_util.log_to_file(config_dic['log_file'])

util_r.source_r_plots(config_dic['source_dir'])
util_r.source_r_clustering(config_dic['source_dir'])


# get baseset
read_scop = scop_sets.ReadScopBaseset(config_dic['min_set_len'], config_dic['max_set_len'], config_dic['atom_sel'])
read_scop.load_file(config_dic['sets_file'])
baseset = read_scop.get_baseset(config_dic['pdb_dir'], config_dic['use_pdb_subdir'])
baseset.set_working_to_current_dir()
set_lst = baseset.get_sentry_id_lst()

# get aligned distance array
alig_file = os.path.join(os.environ['STRUSTER'],'DATA/SETS/51459_subsetAB_alig.fasta')
baseset.get_aligment_from_msa(alig_file)
baseset.get_alig_dist_array()
baseset.get_alig_gc_dist_array()
baseset.get_alig_sus(dpi_file = config_dic['dpi_file'])
baseset.get_alig_gc_sus(dpi_file = config_dic['dpi_file'])

# define ligand binding site, from d1pig_2
site_sentry_id = 'd1pig_2'
site_reskey_lst = [' :58 ', ' :59 ', ' :60 ', ' :61 ', ' :62 ', ' :63 ', ' :98 ', ' :101 ', ' :103 ', ' :104 ', ' :105 ', ' :151 ', ' :162 ', ' :163 ', ' :164 ',' :165 ', ' :195 ', ' :197 ', ' :198 ', ' :199 ', ' :200 ', ' :201 ', ' :233 ', ' :235 ', ' :236 ', ' :237 ', ' :238 ', ' :240 ', ' :298 ', ' :299 ', ' :300 ', ' :305 ', ' :306 ', ' :307 ', ' :308 ']

# clustering
clusterset =  clustering.ClusterSet(baseset, config_dic['dist_filter'], config_dic['dif_dist_filter'], config_dic['one_gap_value'], config_dic['two_gap_value'], config_dic['get_superp'], config_dic['cl_atoms'])
clusterset.cluster_subdmat(site_sentry_id, site_reskey_lst)
clusterset.R_h_sil()
clusterset.R_pam_cluster()

# variation
# S GC
s_mat = baseset.get_dist_sdev_mat(set_lst, 'GC')
baseset.plot_sdev(s_mat, '51459_subsetAB_S_GC')
# ligand binding site
submat = baseset.get_submat(site_sentry_id, site_reskey_lst, s_mat)
baseset.plot_sdev(submat, '51459_subsetAB_ligsite_S_GC')

# R GC
nmat = baseset.get_sqrtmean_dsusv_mat(set_lst, 'GC')
r_mat = baseset.get_rel_dist_sdev_mat(s_mat, nmat)
baseset.plot_relsdev(r_mat, '51459_subsetAB_R_mean_GC')
# ligand binding site
submat = baseset.get_submat(site_sentry_id, site_reskey_lst, r_mat)
baseset.plot_relsdev(submat, '51459_subsetAB_ligsite_R_mean_GC')


# T GC
t_mat = baseset.get_dist_sus_sdev_mat(set_lst, s_mat, 'GC')
baseset.plot_sdev(t_mat, '51459_subsetAB_T_GC')
# hinges/invariant
und_mat = baseset.get_undef_mat(t_mat)
hi_obj = hinge_inv.HingeInv('51459_subsetAB_T_1:0.1:0.2_GC', config_dic['source_dir'], '.', True)
hi_obj.get_hinges_invariant(score_mat = t_mat, undef_mat = und_mat, score_cutoff = 1.0, filter_cutoff = 0.1, min_gap = 10, inv_cutoff =0.2, mat_type = 'set')
# ligand binding site
submat = baseset.get_submat(site_sentry_id, site_reskey_lst, t_mat)
baseset.plot_sdev(submat, '51459_subsetAB_ligsite_T_GC')


# X GC
x_mat = baseset.get_x_mat(set_lst, 'GC')
baseset.plot_relsdev(x_mat, '51459_subsetAB_X_GC')
# ligand binding site
submat = baseset.get_submat(site_sentry_id, site_reskey_lst, x_mat)
baseset.plot_relsdev(submat, '51459_subsetAB_ligsite_X_GC')


# finish logging
log_util.close_log()
