# Copyright 2007 Max-Planck-Institut Informatik
# 
#     This file is part of STRuster.
# 
#     STRuster is free software; you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation; either version 2 of the License, or
#     (at your option) any later version.
# 
#     STRuster is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
# 
#     You should have received a copy of the GNU General Public License
#     along with STRuster; if not, write to the Free Software
#     Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
#
# Contact:
#     Francisco S. Domingues
#     email: doming@mpi-sb.mpg.de


library(mva)
library(cluster)



h.cluster.dend = function(dist.filename, out.filename, titleplot, imgw, imgh, imgpsz ) {
  dmat = as.matrix(read.table(file= dist.filename))
  cdist = as.dist(dmat)
  h = hclust(cdist,method="average")
  png(out.filename, width = imgw, height = imgh, pointsize = imgpsz)
  plclust(h, main = titleplot,ylab= '', xlab = '', sub = '')
  dev.off()
}



h.cluster = function(dfilename,aswfilename,silfilename,setname,max.clus=1000) {
  # asw plot
  distmat = read.table(dfilename)
  kmin = 2
  kmax = min(max.clus,ncol(distmat)-1)
  asw.hc = rep(0,kmax)
  hc = hclust(as.dist(distmat),method="average")
  for (k in c(kmin:kmax)) {
    sil.hc = silhouette(cutree(hc,k),distmat)
    # sil.hc[which(sil.hc[,3]==1),3] = 0
    asw.hc[k] = mean(sil.hc[,3]) 
  }
  png(aswfilename)
  xlab = "number of clusters"
  ylab = "average silhouette width"
  plot(asw.hc,lwd=2,type="l",lty="solid",ylim=c(0,1),xlab=xlab,ylab=ylab,cex=1.1,cex.lab=1.1)
  abline(v=which.max(asw.hc),lwd=2,lty="dotted")
  title(paste(setname,":  Average silhouette widths for HC",sep=""))
  dev.off()
  # silhouette plot
  hc.best = cutree(hc,which.max(asw.hc))
  sil.hc = silhouette(hc.best,distmat)
  # singletons = sil.hc[,1]%in%which(table(sil.hc[,1])==1)
  # sil.hc[singletons,3] = 0
  png(silfilename, width = 1024, height = 1024)
  main = paste(setname,":  Silhouette plot for HC\n",sep="")
  order = order(sil.hc[,"cluster"],-sil.hc[,"sil_width"])
  dimnames(sil.hc)[[1]] = dimnames(sil.hc)[[1]][order]
  plot(sil.hc,do.clus.stat=F,main=main,nmax.lab=100,max.strlen=8)
  dev.off()
  list(silhouettes=sil.hc[,c(1,3)],average.silhouette.value=mean(sil.hc[,3]), nrclusters = which.max(asw.hc))
}




pam.cluster = function(dfilename,aswfilename,silfilename,setname,max.clus=1000) {
  # asw plot
  distmat = read.table(dfilename)
  kmin = 2
  kmax = min(max.clus,ncol(distmat)-1)
  asw.pam = rep(0,kmax)
  for (k in c(kmin:kmax)) {
    asw.pam[k] = pam(as.dist(distmat),k) $ silinfo $ avg.width 
  }
  png(aswfilename)
  xlab = "number of clusters"
  ylab = "average silhouette width"
  plot(asw.pam,lwd=2,type="l",lty="solid",ylim=c(0,1),xlab=xlab,ylab=ylab,cex=1.1,cex.lab=1.1)
  abline(v=which.max(asw.pam),lwd=2,lty="dotted")
  title(paste(setname,":  Average silhouette widths for PAM",sep=""))
  dev.off()
  # silhouette plot
  pam.best = pam(as.dist(distmat),which.max(asw.pam))
  sil.pam = silhouette(pam.best$clustering,distmat)
  # need to check sil.pam is NA if there is only one cluster
  if (is.na(sil.pam) ) {
    png(silfilename, width = 1024, height = 1024)
    main = paste(setname,":  Silhouette plot for PAM\n",sep="")
    barplot(c(0), main=main)
    dev.off()
    list(nrclusters = 1)      
  }
  else {
    # singletons = sil.pam[,1]%in%which(table(sil.pam[,1])==1)
    # sil.pam[singletons,3] = 0
    png(silfilename, width = 1024, height = 1024)
    main = paste(setname,":  Silhouette plot for PAM\n",sep="")
    order = order(sil.pam[,"cluster"],-sil.pam[,"sil_width"])
    dimnames(sil.pam)[[1]] = dimnames(sil.pam)[[1]][order]
    # replace nan by 1.0
    # need to uncomment
    # innirvana = is.nan(sil.pam[,3])
    # sil.pam[innirvana,3] = 1.0
    plot(sil.pam,do.clus.stat=F,main=main,nmax.lab=100,max.strlen=10, cex =0.8, xlab = 'Silhouette width')
    dev.off()
    list(silhouettes=sil.pam[,c(1,3)],average.silhouette.value=mean(sil.pam[,3]),nrclusters = which.max(asw.pam),medoids=pam.best$medoids)
  } 
}




rand.index = function(classlabel1,classlabel2) {
  table = table(classlabel1,classlabel2)
  n = length(classlabel1)
  ni = apply(table,1,sum)
  nj = apply(table,2,sum)
  sum = choose(n,2)
  sumi = sum(choose(ni[ni>1],2))
  sumj = sum(choose(nj[nj>1],2))
  sumij = sum(choose(as.vector(table[table>1]),2))
  rand = (sumij-(sumi*sumj)/sum)/(1/2*(sumi+sumj)-(sumi*sumj)/sum)
  if (is.nan(rand)) {
  rand = 1.0
  }
  rand
}
