# Copyright 2007 Max-Planck-Institut Informatik
# 
#     This file is part of STRuster.
# 
#     STRuster is free software; you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation; either version 2 of the License, or
#     (at your option) any later version.
# 
#     STRuster is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
# 
#     You should have received a copy of the GNU General Public License
#     along with STRuster; if not, write to the Free Software
#     Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
#
# Contact:
#     Francisco S. Domingues
#     email: doming@mpi-sb.mpg.de



read.matrices = function(name.cb, name.dif) {
  d1 = as.matrix(read.table(file=name.cb))
  d2 = as.matrix(read.table(file=name.dif))
  n1 = dim(d1)[1]/dim(d1)[2]
  n2 = dim(d2)[1]/dim(d2)[2]
  nr = dim(d1)[2]
  list(d1=d1,d2=d2,n1=n1,n2=n2,nresidues=nr) }


# Comparing two sets

test2sets = function(input, test.type) {
  nr = input$nresidues
  d1 = input$d1
  d2 = input$d2
  n1 = input$n1
  n2 = input$n2
  fp.pvalue = matrix(0,nrow=nr,ncol=nr)
  for (i in c(1:(nr-1))) {
    plot(i)
    for (j in c((i+1):nr)) {
      d1vec = as.numeric(d1[i+c(0:(n1-1))*nr,j])
      d2vec = as.numeric(d2[i+c(0:(n2-1))*nr,j])
      length.d1vec = length(d1vec[!is.na(d1vec)])
      length.d2vec = length(d2vec[!is.na(d2vec)])
      m = min(length.d1vec,length.d2vec)
      if (m>=2) {
        if (test.type == "wilcox.paired") {
          test = wilcox.test(d1vec,d2vec,paired=F) }
        if (test.type == "wilcox") {
          test = wilcox.test(d1vec,d2vec) }
        if (test.type == "t") {
          test = t.test(d1vec,d2vec,paired=F) }
        pvalue = test$p.value }
      if (m<2) { pvalue = -1 }
      fp.pvalue[i,j] = pvalue
      fp.pvalue[j,i] = pvalue }
      fp.pvalue[i,i] = 1 }
  fp.pvalue[nr,nr] = 1
  list(fp.pvalue=fp.pvalue) }


plot.test2sets = function(matrix, p0=0.01, name="name", plot.type="png") {
  missing = which(matrix<0)
  matrix = matrix<p0
  matrix[missing]=-1
  nr = dim(matrix)[2]
  if (plot.type=="png") {
    png(name, width = 1024, height = 1024)
    filled.contour(matrix,x=c(1:nr),y=c(1:nr),zlim=c(-1,1))
    dev.off() }
  else {
    postscript(name, onefile=F,horizontal=F,width=7,height=7)
    filled.contour(matrix,x=c(1:nr),y=c(1:nr),zlim=c(-1,1)) 
    dev.off()}}


compsets = function(nameset1, nameset2, outname, p0=0.01,  p.adjust = "none", test.type="t", plot.type="png") {
  input = read.matrices(nameset1, nameset2)
  testrsl = test2sets(input, test.type)
  if (p.adjust == "bonf") {
     plot.test2sets(matrix=testrsl$fp.pvalue, p0/choose(dim(input$d2)[1],2) , outname, plot.type) }
  if (p.adjust == "none") {
     plot.test2sets(matrix=testrsl$fp.pvalue, p0 , outname, plot.type) }}




# comparing query to set

testquery = function(input) {
  nr = input$nresidues
  d1 = input$d1
  d2 = input$d2
  n1 = input$n1
  n2 = input$n2
  fp.pvalue = matrix(0,nrow=nr,ncol=nr)
  fp.mean = matrix(0,nrow=nr,ncol=nr)
  fp.sd = matrix(0,nrow=nr,ncol=nr)
  fp.range = matrix(0,nrow=nr,ncol=nr)
  for (i in c(1:(nr-1))) {
    # plot(i)
    for (j in c((i+1):nr)) {
      d1vec = d1[i+c(0:(n1-1))*nr,j]
      d2val = d2[i,j]
      m = mean(d1vec)
      sd = sd(d1vec)
      fp.mean[i,j] = m
      fp.mean[j,i] = m
      fp.sd[i,j] = sd
      fp.sd[j,i] = sd
      pvalue = pt(-abs((d2val-m)/sd),(n1-1))*2
      fp.pvalue[i,j] = pvalue
      fp.pvalue[j,i] = pvalue
      min = min(d1vec)-1/(n1-1)*(max(d1vec)-min(d1vec))
      min = max(min,0)
      max = max(d1vec)+1/(n1-1)*(max(d1vec)-min(d1vec))
      if ((d2val<=max) & (d2val>=min)) { r = 0 }
      if (d2val>max) { r = (d2val-max)/(max-min) }
      if (d2val<min) { r = (min-d2val)/(max-min) }
      fp.range[i,j] = r
      fp.range[j,i] = r }
      fp.pvalue[i,i] = 1 }
  fp.pvalue[nr,nr] = 1
  list(fp.pvalue=fp.pvalue,fp.mean=fp.mean,fp.sd=fp.sd,fp.range=fp.range) }


plot.testquery = function(matrix, missing, p0=0.01, name="name", plot.type="png") {
  matrix = matrix<p0
  matrix[missing] = -1
  nr = dim(matrix)[1]
  if (plot.type=="png") {
     png(name, width = 1024, height = 1024)
     filled.contour(matrix,x=c(1:nr),y=c(1:nr),zlim=c(-1,1))
     dev.off() }
  else {
     postscript(paste(name,".dist.eps",sep=""),onefile=F,horizontal=F,width=7,height=7)
     filled.contour(matrix,x=c(1:nr),y=c(1:nr),zlim=c(-1,1))
     dev.off() }}

compqueryset = function(namequery, nameset, protname, p0=0.01, p.adjust = "none", plot.type="png") {
  input = read.matrices(nameset,namequery)	     
  testrsl = testquery(input)
  if (p.adjust == "bonf") {
    plot.testquery(matrix=testrsl$fp.pvalue, missing=input$d2<0, p0/choose(dim(input$d2)[1],2), protname, plot.type) }
  if (p.adjust == "none") {
    plot.testquery(matrix=testrsl$fp.pvalue, missing=input$d2<0, p0, protname, plot.type) }}


