"""
STRuster web site
"""

# Copyright 2007 Max-Planck-Institut Informatik
# 
#     This file is part of STRuster.
# 
#     STRuster is free software; you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation; either version 2 of the License, or
#     (at your option) any later version.
# 
#     STRuster is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
# 
#     You should have received a copy of the GNU General Public License
#     along with STRuster; if not, write to the Free Software
#     Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
#
# Contact:
#     Francisco S. Domingues
#     email: doming@mpi-sb.mpg.de



import os
import cPickle

import log_util
import scop_cla



class STRusterHTML:
    """
    Generate the HTML pages of STRUster results
    """
    
    def __init__(self, scop_des_file, scop_cla_file, rslhome_dir, version_nr):
    
        self.scop_des_file = scop_des_file
        self.scop_cla_file = scop_cla_file
        self.rslhome_dir = rslhome_dir
        self.version_nr = version_nr
        self.des_dic = {}  # des_dic[sunid] SCOP description
        self.cla_dic = {}  # cla_dic[sid] sentry_dic, sentry_dic[key]value: sid , pdb, dom_def, and sunids for cl, cf, sf, fa, dm, sp, px
        self.sp_dic = {}     # dictionary with descrition of species set sp_dic[sp_sunid] SpeciesDes_obj
        self.rsl_lst = []   # lists of result directories
        
        
        
    def get_struster_html(self):
        self.get_scop_data()
        self.get_rsl_lst()
        self.make_html()

        
        
                    
    def get_scop_data(self): 
        self.des_dic = scop_cla.get_scop_des(self.scop_des_file)
        self.cla_dic = scop_cla.get_scop_cla(self.scop_cla_file)
        for sid in self.cla_dic.keys():
            sp_sunid = self.cla_dic[sid]['sp']
            if self.sp_dic.has_key(sp_sunid):
                self.sp_dic[sp_sunid].add_sentry(sid)
            else:
                sp_obj = SpeciesDes(sid)
                sp_obj.cl = self.get_des('cl', sid)
                sp_obj.cf = self.get_des('cf', sid)
                sp_obj.sf = self.get_des('sf', sid)
                sp_obj.fa = self.get_des('fa', sid)
                sp_obj.dm = self.get_des('dm', sid)
                sp_obj.sp = self.get_des('sp', sid)
                self.sp_dic[sp_sunid] = sp_obj
        log_util.log_info("Loaded SCOP data")
                
                
        
    def get_des(self, scop_level, sid):
        sunid = self.cla_dic[sid][scop_level]
        scop_des = self.des_dic[sunid]
        return scop_des
        
        
        
    def get_rsl_lst(self):
        out_lst = os.listdir(self.rslhome_dir)
        for rsl_dir in out_lst:
            if rsl_dir[-6:] == '_sunid':
                self.rsl_lst.append(rsl_dir)
        self.rsl_lst.sort()
        log_util.log_info("Got %d result directories %s" % (len(self.rsl_lst), self.rsl_lst))       
        
        
        
    def make_html(self):
        for rsl_dir in self.rsl_lst:
            os.chdir(self.rslhome_dir)
            os.chdir(rsl_dir)
            sp_sunid = rsl_dir[:-6]
            sp_obj = self.sp_dic[sp_sunid]
            rsl_obj =  self.get_data_files(sp_sunid, sp_obj.nr_sentries)
            if  rsl_obj == None:
                log_util.log_warning("Incomplete results for set %s" % sp_sunid)
                continue
            rsl_obj.read_hing_inv()
            rsl_obj.rotate_dend()
            rsl_obj.write_html(sp_obj)
            # to do: save data            
        os.chdir(self.rslhome_dir)
        self.make_rsl_index()
            
        
            
    def get_data_files(self, sp_sunid, nr_sentries):
        rsl_obj = STRusterRsl(sp_sunid, self.version_nr)
        if rsl_obj.check_required(nr_sentries) == True:
            return rsl_obj
        else:
            return None
    
            
                
    def make_rsl_index(self):
        rsl_index_file = 'rsl_list_'+ self.version_nr + '.html'
        fobj = file(rsl_index_file, 'w')
        # head
        fobj.write('<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">\n')
        fobj.write('<html><head>\n')
        fobj.write('<meta http-equiv="Content-Type" content="text/html; charset=ISO-8859-1" />\n')
        out_str = 'Index of STRuster results'
        fobj.write('<title>' + out_str + '</title>\n')
        fobj.write('</head><body>\n\n')

        # title
        out_str='Index of STRuster results with SCOP sunid codes'
        fobj.write('<div style="font-family:arial sans-serif; font-size:18px"><b><a href="index.php">' + 'STRuster' + '</a></b></div><br /><br />\n')
        fobj.write('<table cellpadding="2" cellspacing="0" border="0" width="100%">\n<tr>\n')
        fobj.write('<td style="background:#aac1d6; font-family:arial sans-serif; font-size:18px">\n')
        fobj.write('<b>' + out_str + '</b></td></tr>\n</table>\n<br /><br />\n')
        
        for rsl_dir in self.rsl_lst:
            sunid = rsl_dir[:-6]
            result_filename = sunid + '.html'
            result_file = os.path.join(rsl_dir,  result_filename)
            fobj.write('<a href="' + result_file + '">' + sunid + '</a> ')
        fobj.write('</body></html>\n')
        fobj.close()
        log_util.log_info("Created HTML: %s" % rsl_index_file)
    
    
                
class STRusterRsl:

    def __init__(self, sp_sunid, version_nr):
        self.sp_sunid = sp_sunid
        self.version_nr = version_nr
        self.html_filename = sp_sunid + '.html'
        self.alig_filename = sp_sunid + '.alig'
        self.aligfasta_filename = sp_sunid + '_alig.fasta'
        self.cluster_dend_filename = sp_sunid + '_cluster_dend.png'   # not required
        self.cluster_dend_r_filename = sp_sunid + '_cluster_dend_r.png'   # not required
        self.cluster_dmat_filename = sp_sunid + '_dissim_matrix.txt'
        self.hinge_inv_filename = sp_sunid + '_hinge_inv.data'
        self.dist_sdev_filename = sp_sunid + '_set_dist_sdev.png'
        self.dist_sus_sdev_filename = sp_sunid + '_set_dist_sus_sdev.png'
        self.rel_dist_sdev_filename = sp_sunid + '_set_rel_dist_sdev_mean.png'
        self.superp1_filename = sp_sunid + '_superp_1.pdb.gz' # not required
        self.superp2_filename = sp_sunid + '_superp_2.pdb.gz' # not required
        self.superp3_filename = sp_sunid + '_superp_3.pdb.gz' # not required
        self.superp_filename_lst = [self.superp1_filename, self.superp2_filename, self.superp3_filename]
        self.rasm1_filename = sp_sunid + '_superp_1.rsm' # not required
        self.rasm2_filename = sp_sunid + '_superp_2.rsm' # not required
        self.rasm3_filename = sp_sunid + '_superp_3.rsm' # not required        
        self.rasm_filename_lst = [self.rasm1_filename, self.rasm2_filename, self.rasm3_filename]
        self.hinge_inv_dic = None
        
        
        
    def check_required(self, nr_sentries):
        # checking required files
        required_lst = [self.alig_filename, self.aligfasta_filename, self.cluster_dmat_filename, self.hinge_inv_filename, self.dist_sdev_filename, self.dist_sus_sdev_filename, self.rel_dist_sdev_filename ]
        for required_file in required_lst:
            ret_val = os.path.isfile(required_file)
            if ret_val != True:
                log_util.log_warning("Missing %s" % required_file)
                return False
        # checking clustering results    
        if os.path.isfile(self.cluster_dend_filename) != True:
            if nr_sentries >2:
                log_util.log_warning("Sunid %s, no clustering results with %d entries" % (self.sp_sunid, nr_sentries))        
        return True
        
        
        
    def read_hing_inv(self):
        fobj = file(self.hinge_inv_filename, 'r')
        self.hinge_inv_dic = cPickle.load(fobj)
        fobj.close()
        
        
        
    def rotate_dend(self):
        if os.path.isfile(self.cluster_dend_filename):
            cmd_str = "convert -rotate 90 %s %s" % (self.cluster_dend_filename, self.cluster_dend_r_filename)
            os.system(cmd_str)
            log_util.log_info("Created %s" % self.cluster_dend_r_filename)
            
            
            
    def write_html(self, sp_obj):
        fobj = file(self.html_filename, 'w')
    
        # head
        fobj.write('<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">')
        fobj.write('<html>')
        out_str = 'STRuster results: ' + self.sp_sunid
        fobj.write('<head>\n')
        fobj.write('<META HTTP-EQUIV="Content-Type" CONTENT="text/html; charset=ISO-8859-1">')
        fobj.write('<title>' + out_str + '</title>\n')
        fobj.write('</head><body>\n\n')
        
        # title
        fobj.write('<font face="arial" size="4"><b><a href="../index.php">' + 'STRuster' + '</a></b></font><p>\n')
        fobj.write('<table cellpadding="2" cellspacing="0" border="0" width="100%">\n<tr>\n')
        fobj.write('<td bgcolor="#aac1d6">&nbsp;<font face="arial"\n')
        fobj.write('    size="4"><b>' + out_str + '</b></font>\n</td></tr>\n</table>\n<p>\n')
        
        # scop classification
        fobj.write('<table cellpadding="2" cellspacing="0" border="0" width="100%">\n<tr>\n')
        fobj.write('<td bgcolor="#DEB887">&nbsp;<font face="arial"\n')
        fobj.write('    size="3"><b>SCOP 1.71 classification</b></font>\n</td></tr>\n</table>\n')
        fobj.write('<table>\n')
        fobj.write('<tr><td align="left">Class:</td>\n')
        fobj.write('    <td align="left">' + sp_obj.cl + '</td>\n</tr>\n')
        fobj.write('<tr><td align="left">Fold:</td>\n')
        fobj.write('    <td align="left">' + sp_obj.cf + '</td>\n</tr>\n')
        fobj.write('<tr><td align="left">Superfamily:</td>\n')
        fobj.write('    <td align="left">' + sp_obj.sf + '</td>\n</tr>\n')
        fobj.write('<tr><td align="left">Family:</td>\n')
        fobj.write('    <td align="left">' + sp_obj.fa + '</td>\n</tr>\n')
        fobj.write('<tr><td align="left">Protein:</td>\n')
        fobj.write('    <td align="left">' + sp_obj.dm + '</td>\n</tr>\n')
        fobj.write('<tr><td align="left">Species:</td>\n')
        fobj.write('    <td align="left"><i>' + sp_obj.sp + '</i></td>\n</tr>\n')
        fobj.write('<tr><td align="left">Species SCOP sunid:</td>\n')
        fobj.write('    <td align="left">' + self.sp_sunid + '</td>\n</tr>\n</table>\n\n<p>\n\n')
        
        # set
        fobj.write('<table cellpadding="2" cellspacing="0" border="0" width="100%">\n<tr>\n')
        fobj.write('<td bgcolor="#DEB887">&nbsp;<font face="arial"\n')
        fobj.write('    size="3"><b>Set description</b></font>\n</td></tr>\n</table>\n\n')
        fobj.write('<table>\n')
        fobj.write('<tr><td align="left">Number of entries:</td>\n')
        fobj.write('    <td align="left">' + str(sp_obj.nr_sentries) + '</td>\n</tr>\n')
        fobj.write('<tr><td align="left">SCOP Entries:</td>\n')
        out_str = ''
        for sid in sp_obj.sid_lst:
            out_str = out_str + sid + ' '
        fobj.write('    <td align="left">'+ out_str + '</td>\n</tr>\n')
        fobj.write('<tr><td align="left">Alignment File:</td>\n')
        fobj.write('    <td align="left"><a href="' + self.alig_filename + '">' + self.alig_filename + '</a></td>\n</tr>\n')
        fobj.write('<tr><td align="left">Aligned Sequences:</td>\n')
        fobj.write('    <td align="left"><a href="' + self.aligfasta_filename + '">' + self.aligfasta_filename + '</a></td>\n</tr>\n')        
        fobj.write('</table>\n\n<p>\n\n')
                        
        # clustering
        fobj.write('<table cellpadding="2" cellspacing="0" border="0" width="100%">\n<tr>\n')
        fobj.write('<td bgcolor="#DEB887">&nbsp;<font face="arial"\n')
        fobj.write('    size="3"><b>Hierarchical clustering</b></font>\n</td></tr>\n</table>\n')
        if os.path.isfile(self.cluster_dend_r_filename) == True:
            fobj.write('<img src="' + self.cluster_dend_r_filename + '" ALT="Dendrogram">\n\n<p>\n')
        fobj.write('\n<table>\n')
        fobj.write('<tr><td align="left">Dissimilary matrix:</td>\n')
        fobj.write('    <td align="left"><a href="' + self.cluster_dmat_filename + '">' + self.cluster_dmat_filename + '</a></td>\n</tr>\n</table>\n\n<p><p>\n\n')
        
        # dist sdev
        fobj.write('<table cellpadding="2" cellspacing="0" border="0" width="100%">\n<tr>\n')
        fobj.write('<td bgcolor="#DEB887">&nbsp;<font face="arial"\n')
        fobj.write('    size="3"><b>Atom Distance Standard Deviation Plots</b></font>\n</td></tr>\n</table>\n')
        fobj.write('<table cellpadding="2" cellspacing="0" border="0" width="100%">\n<tr>\n')
        fobj.write('<td bgcolor="#dcdcdc">&nbsp;<font face="arial"\n')
        fobj.write('    size="2">Distance Standard Deviation</font>\n</td></tr>\n</table>\n')        
        fobj.write('<img src="' + self.dist_sdev_filename + '" ALT="distance standard deviation">\n\n<p>\n')
        fobj.write('<table cellpadding="2" cellspacing="0" border="0" width="100%">\n<tr>\n')
        fobj.write('<td bgcolor="#dcdcdc">&nbsp;<font face="arial"\n')
        fobj.write('    size="2">Total Standard Deviation</font>\n</td></tr>\n</table>\n')        
        fobj.write('<img src="' + self.dist_sus_sdev_filename + '" ALT="distance+sus standard deviation">\n\n<p>\n')
        fobj.write('<table cellpadding="2" cellspacing="0" border="0" width="100%">\n<tr>\n')
        fobj.write('<td bgcolor="#dcdcdc">&nbsp;<font face="arial"\n')
        fobj.write('    size="2">Relative Distance Standard Deviation</font>\n</td></tr>\n</table>\n')        
        fobj.write('<img src="' + self.rel_dist_sdev_filename + '" ALT="relative distance standard deviation">\n\n<p>\n')
        
        # hinges and invariant regions
        fobj.write('<table cellpadding="2" cellspacing="0" border="0" width="100%">\n<tr>\n')
        fobj.write('<td bgcolor="#DEB887">&nbsp;<font face="arial"\n')
        fobj.write('    size="3"><b>Hinges and Invariant Regions</b></font>\n</td></tr>\n</table>\n')
        if len(self.hinge_inv_dic['hinges_lst']) >= 1:
            fobj.write('<table cellpadding="2" cellspacing="0" border="0" width="100%">\n<tr>\n')
            fobj.write('<td bgcolor="#dcdcdc">&nbsp;<font face="arial"\n')
            fobj.write('    size="2">Hinges</font>\n</td></tr>\n</table>\n')
            fobj.write('<table>\n')
            fobj.write('<tr><td align="left">Alignment Nr</td><td align="left">' + str(self.hinge_inv_dic['hinges_lst']) + '</td></tr>\n')
            fobj.write('<tr><td align="left">Uniprot Nr</td>  <td align="left">' + str(self.hinge_inv_dic['hinges_resnr_lst']) + '</td></tr>\n')
            fobj.write('</table>\n\n<p>\n\n')
        total_nr_regions = len(self.hinge_inv_dic['inv_region_lst'])            
        if total_nr_regions >= 1:
            fobj.write('<table cellpadding="2" cellspacing="0" border="0" width="100%">\n<tr>\n')
            fobj.write('<td bgcolor="#dcdcdc">&nbsp;<font face="arial"\n')
            fobj.write('    size="2">Invariant Regions</font>\n</td></tr>\n</table>\n\n')
            fobj.write('<table>\n')
            for region_count in range(total_nr_regions):
                inv_reg = self.hinge_inv_dic['inv_region_lst'][region_count]
                inv_reg_resnr = self.hinge_inv_dic['inv_region_resnr_lst'][region_count]                
                fobj.write('<tr><td align="left">Region</td><td align="left">' + str(region_count + 1) + '</td></tr>\n')
                fobj.write('<tr><td align="left">Alignment Nr</td><td align="left">' + str(inv_reg) + '</td></tr>\n')
                fobj.write('<tr><td align="left">Uniprot Nr</td><td align="left">' + str(inv_reg_resnr) + '</td></tr>\n')
                if region_count < len(self.superp_filename_lst):
                    if os.path.isfile(self.superp_filename_lst[region_count]) == True:
                        superp_filename = self.superp_filename_lst[region_count]
                        fobj.write('<tr><td align="left">Superposition</td><td align="left"><a href="' + superp_filename + '">' + superp_filename + '</a></td></tr>\n')
                        if os.path.isfile(self.rasm_filename_lst[region_count]) == True:
                            rasm_filename = self.rasm_filename_lst[region_count]
                            fobj.write('<tr><td align="left">Rasmol Script</td><td align="left"><a href="' + rasm_filename + '">' + rasm_filename + '</a></td></tr>\n')    
                            jmol_html_file = self._write_jmol_html(self.sp_sunid, (region_count+1), rasm_filename)
                            fobj.write('<tr><td align="left"><a href="' + jmol_html_file + '" target ="_blank">' + "View in Jmol" + '</a></td><td></td></tr>\n')
                    else:
                        fobj.write('<tr><td align="left">Superposition</td><td align="left">-</td></tr>\n')
                else:
                    fobj.write('<tr><td align="left">Superposition</td><td align="left">-</td></tr>\n')
                fobj.write('<tr><td>&nbsp</td></tr>\n')
            fobj.write('</table>\n\n')        
        
        
        # end
        fobj.write('</body></html>\n')
        fobj.close()
        log_util.log_info("Created HTML: %s" % self.html_filename)
        
        self.make_link()
        
        
        
    def make_link(self):
        cmd_str = "ln -f -s " + self.html_filename + " index.html"
        os.system(cmd_str)
        
    def _write_jmol_html(self, sp_sunid, region_nr, rasm_filename):
        jmol_html_file = "jmol" + str(region_nr) +".html"
        fobj = file(jmol_html_file, 'w')
        fobj.write('<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">\n')
        fobj.write('<html>\n')
        fobj.write('<head>\n')
        fobj.write('<meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1">\n')
        fobj.write('<title>%s Superposition %d</title>\n' % (sp_sunid, region_nr))
        fobj.write('<script src="../../../bin/jmol-11.0.1/Jmol.js" type="text/javascript"></script>\n')
        fobj.write('</head><body>\n\n')
        fobj.write(' <form>\n')
        fobj.write('    <script type="text/javascript">\n')
        fobj.write('    jmolInitialize("../../../bin/jmol-11.0.1");\n')
        fobj.write('    jmolApplet(400, "script %s");\n' % rasm_filename)
        fobj.write('    </script>\n')
        fobj.write(' </form>\n')        
        fobj.write('</body></html>\n')
        fobj.close()
        return jmol_html_file
    
                                                    
class SpeciesDes:
    "Species set  descriptions"


    def __init__(self, sid):
        self.cl = None
        self.cf = None
        self.sf = None
        self.fa = None
        self.dm = None
        self.sp = None
        self.nr_sentries = 1
        self.sid_lst = [sid]
    
        
        
    def add_sentry(self, sid):
        self.sid_lst.append(sid)
        self.nr_sentries += 1
        
        
        
        
        
class STRusterStats:
    """
    Compute some final statistics on the STRuster final results
    """
    
    def __init__(self, rslhome_dir, stats_file_name):
        self.rslhome_dir = rslhome_dir
        self.rsl_lst = []
        self.set_id_lst = []
        self.struster_hi_dic = {}
        self.stats_file_name = stats_file_name
        
        
        
    def get_rsl_stats(self):
        # read the data files
        self.get_hi_data()
        # get list of results
        self.get_rsl_lst()
        for rsl_dir in self.rsl_lst:
            os.chdir(self.rslhome_dir)
            os.chdir(rsl_dir)
            sp_sunid = rsl_dir[:-6]
            html_filename = sp_sunid + '.html'
            cluster_dmat_filename = sp_sunid + '_dissim_matrix.txt'
            if os.path.isfile(html_filename) and os.path.isfile(cluster_dmat_filename):
                nr_sentries = self.get_nr_sentries(cluster_dmat_filename)
                if self.struster_hi_dic.has_key(sp_sunid):
                    hi_dic = self.struster_hi_dic[sp_sunid]
                    hi_dic['nr_sentries'] = nr_sentries
                    self.set_id_lst.append(sp_sunid)
                else:
                    log_util.log_warning("No hinge/invariant data for %s" % str(sp_sunid))
            else:
                log_util.log_warning("Results not complete for %s" % sp_sunid)
        os.chdir(self.rslhome_dir)
        self.set_id_lst.sort()
        self.write_stats()


                
    def get_rsl_lst(self):
        out_lst = os.listdir(self.rslhome_dir)
        for rsl_dir in out_lst:
            if rsl_dir[-6:] == '_sunid':
                self.rsl_lst.append(rsl_dir)
        self.rsl_lst.sort()
        log_util.log_info("Got %d result directories" % len(self.rsl_lst))       

        
        
    def get_nr_sentries(self, cluster_dmat_filename):
        fobj = file(cluster_dmat_filename, 'r')
        lines_lst = fobj.readlines()
        fobj.close()
        nr_sentries = 0
        for line_str in lines_lst:
            line_str.strip()
            if len(line_str) > 0:
                nr_sentries += 1
        # subtract first line
        nr_sentries = nr_sentries -1
        return nr_sentries

        
        
    def get_hi_data(self):
        os.chdir(self.rslhome_dir)
        dir_lst = os.listdir(self.rslhome_dir)
        hi_file_lst = []
        for file_name in dir_lst:
            if file_name[:19]== 'struster_hinge_inv_' and file_name[-5:] == '.data':
                hi_file_lst.append(file_name)
        for hi_file in hi_file_lst:
            fobj = file(hi_file, 'r')
            hinge_inv_dic = cPickle.load(fobj)
            fobj.close()
            log_util.log_info("Read %s" % hi_file)
            set_id_lst = hinge_inv_dic.keys()
            for set_id in set_id_lst:
                if self.struster_hi_dic.has_key(set_id):
                    log_util.log_error("Duplicated set %s" % str(set_id))
                else:
                    self.struster_hi_dic[set_id] = hinge_inv_dic[set_id]

        
                
    def write_stats(self):
        fobj = file(self.stats_file_name,'w')
        fobj.write('sunid  nr_sentries hinges_dsdev inv_dsdev var_dsdev hinges_rsdev inv_rsdev var_rsdev hinges_tsdev inv_tsdev var_tsdev\n')
        for set_id in self.set_id_lst:
            hi_dic = self.struster_hi_dic[set_id]
            sunid = set_id
            nr_sentries = hi_dic['nr_sentries']
            hinges_dsdev = len(hi_dic['dsdev_dic']['hinges_lst'])
            inv_dsdev = len(hi_dic['dsdev_dic']['inv_region_lst'])
            var_dsdev = len(hi_dic['dsdev_dic']['variable_lst'])
            hinges_rsdev = len(hi_dic['reldsdev_dic']['hinges_lst'])
            inv_rsdev = len(hi_dic['reldsdev_dic']['inv_region_lst'])
            var_rsdev = len(hi_dic['reldsdev_dic']['variable_lst'])
            hinges_tsdev = len(hi_dic['dsussdev_dic']['hinges_lst'])
            inv_tsdev = len(hi_dic['dsussdev_dic']['inv_region_lst'])
            var_tsdev = len(hi_dic['dsussdev_dic']['variable_lst'])
            fobj.write('%-10s %4d %4d %4d %4d %4d %4d %4d %4d %4d %4d\n'% (sunid, nr_sentries, hinges_dsdev, inv_dsdev, var_dsdev, hinges_rsdev, inv_rsdev, var_rsdev, hinges_tsdev, inv_tsdev, var_tsdev))
        fobj.close()
        log_util.log_info("Wrote %s" % self.stats_file_name)
    