"""
Utilities for logging
From the python 2.3 library reference

First use log_to_file to start a logger to a file
Then use the following functions to report different types of information
log_info
log_warning

In except on a try/except statment use
log_error
to retrieve the exception type, value and traceback

outside except use
raise RuntimeError, log_string
to signal an exception
this should be caught at higher level in try/except

Apply try/except at high level
use raise at low level

TODO:
Set a logger handler to standard output/error if none is configured
So far I check it with a filehandler flag and I use basicConfig (error to stderr)
"""

# Copyright 2007 Max-Planck-Institut Informatik
# 
#     This file is part of STRuster.
# 
#     STRuster is free software; you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation; either version 2 of the License, or
#     (at your option) any later version.
# 
#     STRuster is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
# 
#     You should have received a copy of the GNU General Public License
#     along with STRuster; if not, write to the Free Software
#     Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
#
# Contact:
#     Francisco S. Domingues
#     email: doming@mpi-sb.mpg.de



import sys
import logging
import traceback

# hack, need to find a better way
# flag that filehandler was set
filehandler = 0



def log_to_file(log_file):
    "Set log_file as the file to log messages to"
    global filehandler

    logger = logging.getLogger()
    hdlr = logging.FileHandler(log_file, 'w')
    formatter = logging.Formatter('%(asctime)s %(levelname)s %(message)s')
    hdlr.setFormatter(formatter)
    logger.addHandler(hdlr)
    logger.setLevel(logging.INFO)    # ignore DEBUG
    filehandler = 1



def log_to_default():
    "Make a default logger configuration with basicConfig"
    global filehandler
    logging.basicConfig()
    filehandler = 1
    


def close_log():
    "Close all logging"
    logging.shutdown()



def log_info(log_string):
    "Log info"
    global filehandler
    if filehandler == 0:
        log_to_default()
    logger = logging.getLogger()
    logger.info(log_string)



def log_warning(log_string):
    "Log warning"
    global filehandler
    if filehandler == 0:
        log_to_default()
    logger = logging.getLogger()
    logger.warning(log_string)
    


def log_error(log_string):
    "Log errors, use within an exception"
    global filehandler
    if filehandler == 0:
        log_to_default()
    logger = logging.getLogger()
    exctype, excvalue, traceback_obj = sys.exc_info()            
    if exctype != None:
        # type and value
        logger.error("%s   %s" % (exctype, excvalue))
        # traceback
        tb_list = traceback.format_tb(traceback_obj)
        del traceback_obj    
        for tb_string in tb_list:
            logger.error(tb_string)
    # log string
    logger.error(log_string)



def get_log():
    """
    demo function
    Get the logger object
    """
    global filehandler
    if filehandler == 0:
        log_to_default()
    logger = logging.getLogger()
    return logger


