# Copyright 2007 Max-Planck-Institut Informatik
# 
#     This file is part of STRuster.
# 
#     STRuster is free software; you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation; either version 2 of the License, or
#     (at your option) any later version.
# 
#     STRuster is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
# 
#     You should have received a copy of the GNU General Public License
#     along with STRuster; if not, write to the Free Software
#     Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
#
# Contact:
#     Francisco S. Domingues
#     email: doming@mpi-sb.mpg.de



# library(rgl)


saveobj = function(robj, robjfile) {
    save(robj, file=robjfile, ascii=TRUE)

}

plotcompscore = function(score_file, plot_file, plot_title, score_cutoff) {
    # offset to minimum val to mark undefined values
    undef_offset = 10.0
    # offset to minimum value
    min_offset = undef_offset / 2
    
    # Plot comparison score    
    score_mat = as.matrix(read.table(file=score_file))
    
    # min max
    n = dim(score_mat)[1]
    max_val = max(score_mat, na.rm = TRUE)
    min_val = min(score_mat, na.rm = TRUE)
    # plot breaks
    max_break = ceiling(max_val)
    min_break = floor(min_val) - min_offset     
    # undefined values are set to min_val-offset
    undef_val = min_break - undef_offset
    score_mat[is.na(score_mat)] = undef_val     
    undef_break = undef_val - undef_offset
    breaks_v = c(undef_break, min_break, -score_cutoff, score_cutoff, max_break)
    col_v    = c(         'gray',    'orange' ,     'white',      'steelblue3')
    # make plot
    png(plot_file, width = 1024, height = 1024)
    # par(bg='gray')
    image(seq(0,(n-1)),seq(0,(n-1)), score_mat, breaks= breaks_v, col= col_v, main = plot_title, xlab = 'alignment position', ylab = 'alignment position')
    box()
    dev.off()
}


susboxplot = function(suslst, idlst, filename) {
    # boxplot
    png(file = filename, , width = 1024, height = 1024)
    par(las = 1, mar = c(5, 5, 2, 2))
    boxplot(x = suslst, names = idlst, horizontal = TRUE, xlab = "\305", ylab = "")
    dev.off()
    # summary data
    all_sus_v = c(suslst[1:length(suslst)], recursive = TRUE)
    summary_str = summary(all_sus_v)
    return(summary_str)
}

susboxplotpdf = function(suslst, idlst, filename) {
    # boxplot
    pdf(file = filename, paper="special", width = 10, height = 10)
    par(las = 1, mar = c(5, 5, 2, 2), cex=1.5)
    boxplot(x = suslst, names = idlst, horizontal = TRUE, xlab = "\305", ylab = "")
    dev.off()
    # summary data
    all_sus_v = c(suslst[1:length(suslst)], recursive = TRUE)
    summary_str = summary(all_sus_v)
    return(summary_str)
}


histsdev = function(matfile, plotfile) {
    hbin_w = 0.02
    mat = as.matrix(read.table(file=matfile))
    max_val = max(mat, na.rm = TRUE)
    max_break = ceiling(max_val)
    breaks_v = seq(from = 0.0, to = max_break, by = hbin_w)
    png(plotfile, , width = 1024, height = 1024)
    hist(as.vector(mat[mat>=0.0]), breaks = breaks_v, main = plotfile, xlab = "\305")
    dev.off()
}




        
plotrelsdev = function(matfile, plotfile) {
    # read matrix
    mat = as.matrix(read.table(file=matfile))
    n = dim(mat)[1]
    
    # undefined values set to -1.0
    mat[is.na(mat)] = -1.0
    
    # maximum value
    max_val = max(mat, na.rm = TRUE)
    max_break = ceiling(max_val)        
    
    # define colors and breaks                           
    cuts_v =      c(-2.0,   -0.5,      2.0,       3.0,           4.0,    5.0)
    cuts_col_v =  c(    'gray', 'white', 'moccasin', 'lightsalmon', 'red3', 'red4')
    cuts_legend_v = c("undef", "[0.0-2.0]", "]2.0-3.0]", "]3.0-4.0]", "]4.0-5.0]")
    if (max_break > cuts_v[length(cuts_v)]) {
        breaks_v = c(cuts_v, max_break)
        col_v = cuts_col_v  
        max_break_str = format(round(max_break,1), nsmall = 1)
        legend_v = c(cuts_legend_v, paste("]5.0-", max_break_str,"]", sep = ''))
    } else {
        breaks_v = cuts_v
        col_v = cuts_col_v[1:length(cuts_col_v)-1]
        legend_v = cuts_legend_v
    }
    png(plotfile, width = 1024, height = 1024)
    par(mar = c(7,7,7,7), xpd = NA)
    image(seq(0,(n-1)),seq(0,(n-1)), mat, breaks = breaks_v, col = col_v, main = '', xlab = 'alignment position', ylab = 'alignment position')
    box()
    legend(x=n, y=n, cex=0.8, legend= legend_v, fill = col_v, bty = "n")
    dev.off()
}

plotvarx = function(matfile, plotfile) {
    # read matrix
    mat = as.matrix(read.table(file=matfile))
    n = dim(mat)[1]
    
    # undefined values set to -1.0
    mat[is.na(mat)] = -1.0
    
    # maximum value
    max_val = max(mat, na.rm = TRUE)
    max_break = ceiling(max_val)        
    
    # define colors and breaks                           
    cuts_v =      c(-2.0,   -0.5,      5.0,          7.5,    10.0)
    cuts_col_v =  c(    'gray', 'white', 'lightsalmon', 'red3', 'red4')
    cuts_legend_v = c("undef", "[0.0-5.0]", "]5.0-7.5]", "]7.5-10.0]")
    if (max_break > cuts_v[length(cuts_v)]) {
        breaks_v = c(cuts_v, max_break)
        col_v = cuts_col_v  
        max_break_str = format(round(max_break,1), nsmall = 1)
        legend_v = c(cuts_legend_v, paste("]10.0-", max_break_str,"]", sep = ''))
    } else {
        breaks_v = cuts_v
        col_v = cuts_col_v[1:length(cuts_col_v)-1]
        legend_v = cuts_legend_v
    }
    png(plotfile, width = 1024, height = 1024)
    par(mar = c(7,7,7,7), xpd = NA)
    image(seq(0,(n-1)),seq(0,(n-1)), mat, breaks = breaks_v, col = col_v, main = '', xlab = 'alignment position', ylab = 'alignment position')
    box()
    legend(x=n, y=n, cex=0.8, legend= legend_v, fill = col_v, bty = "n")
    dev.off()
}


plotsdev = function(matfile, plotfile) {
    # plot of sdev matrix
    
    # read matrix
    mat = as.matrix(read.table(file=matfile))
    n = dim(mat)[1]
    
    # undefined values set to -1.0
    mat[is.na(mat)] = -1.0
    
    # maximum value
    max_val = max(mat, na.rm = TRUE)
    max_break = ceiling(max_val)        
    
    # define colors and breaks                           
    cuts_v =      c(-2.0,   -0.5,    0.5,        1.0,            2.0,     3.0)
    cuts_col_v =  c(    'gray', 'white', 'moccasin', 'lightsalmon', 'red3', 'red4')
    cuts_legend_v = c("undef", "[0.0-0.5]", "]0.5-1.0]", "]1.0-2.0]", "]2.0-3.0]")
    if (max_break > cuts_v[length(cuts_v)]) {
        breaks_v = c(cuts_v, max_break)
        col_v = cuts_col_v
        max_break_str = format(round(max_break,1), nsmall = 1)
        legend_v = c(cuts_legend_v, paste("]3.0-", max_break_str,"]", sep = ''))
    } else {
        breaks_v = cuts_v
        col_v = cuts_col_v[1:length(cuts_col_v)-1]
        legend_v = cuts_legend_v
    }
    
    
    png(plotfile, width = 1024, height = 1024)
    # large marings to include legend
    par(mar = c(7,7,7,7), xpd = NA)
    image(seq(0,(n-1)),seq(0,(n-1)), mat, breaks = breaks_v, col = col_v, main = '', xlab = 'alignment position', ylab = 'alignment position')
    box()
    legend(x=n, y=n, cex=0.8, legend= legend_v, fill = col_v, bty = "n")
    dev.off()        
}



plotsurface = function(mat_file) {
    scale_factor = 20
    marks_spacing = 50
    
    mat = as.matrix(read.table(file=mat_file))
    # remove na
    mat[is.na(mat)] = 0.0
    n1 = dim(mat)[1]
    n2 = dim(mat)[2]
    
    # scale mat
    mat_scaled = mat * scale_factor
    
    # get colors vector
    matint100 = round(mat * 100)        
    mat_range = range(matint100)
    mat_rangelen = mat_range[2] - mat_range[1] +1
    color_table = topo.colors(mat_rangelen)
    col_v = color_table[matint100 - mat_range[1] + 1]
    
    # legend
    #l_m = matrix(seq(1:mat_range[2]), byrow = TRUE, nrow = 1)
    #l_breaks_v = seq(1:(length(color_table)+1))
    #image(c(1), seq(1:mat_range[2]), l_m, breaks = l_breaks_v, col = color_table,  xlab = '', ylab = '' )
    nr_marks = floor((mat_range[2]-mat_range[1]+1)/marks_spacing)
    legend_marks = seq(from=1, to = nr_marks*marks_spacing +1, by = marks_spacing)
    legend_v = rep(NA, mat_rangelen)
    offset = mat_range[1] - legend_marks[1]
    legend_v[legend_marks] = format( round((legend_marks+offset)/100,1), nsmall =1 )
    par(bg = "black", col.axis = "white")
    barplot(rep(1,mat_rangelen), space=c(0,0), horiz = FALSE, border = NA , col = color_table, names.arg=legend_v, las = 1, yaxt = "n")
    
    # get axis marks and labels
    # rows
    nr_marks = floor(n1/marks_spacing)
    row_marks = seq(from =0, to = nr_marks*marks_spacing, by = marks_spacing)
    row_marks[1] = 1
    row_labels = row_marks
    # columns
    nr_marks = floor(n2/marks_spacing)
    col_marks = seq(from =0, to = nr_marks*marks_spacing, by = marks_spacing)
    col_marks[1] = 1
    col_labels = col_marks
    # height axis
    max_h = floor(max(mat, na.rm = TRUE))
    h_labels = seq(from = 0 , to = max_h)
    h_marks = h_labels * scale_factor
        
    rgl.open()
    # bg3d("white")
    rgl.surface(seq(0,(n1-1)), seq(0,(n2-1)), mat_scaled, color = col_v)
    rgl.bbox(xat=row_marks, xlab = row_labels, zat=col_marks, zlab =col_labels, yat=h_marks, ylab = h_labels)
}
