"""
Do some Stats on PDB entries, based on dpi files
"""

# Copyright 2007 Max-Planck-Institut Informatik
# 
#     This file is part of STRuster.
# 
#     STRuster is free software; you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation; either version 2 of the License, or
#     (at your option) any later version.
# 
#     STRuster is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
# 
#     You should have received a copy of the GNU General Public License
#     along with STRuster; if not, write to the Free Software
#     Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
#
# Contact:
#     Francisco S. Domingues
#     email: doming@mpi-sb.mpg.de



import Numeric
import rpy
import sus_dpi

def sum_years(dpi_file):
    
    fobj = file(dpi_file, 'r')
    lines = fobj.readlines()
    pdb_dic = {}
    sigatm_dic = {}
    luzzati_dic = {}
    sigmaa_dic = {}
    dpi_dic = {}
    for line_str in lines:
        line_str = line_str.strip()
        if line_str[0] == '#':
            continue
        line_tpl = eval(line_str)
        pdb_id = line_tpl[0]
        dpi_obj = sus_dpi.DPI(pdb_id)
        dpi_obj.get_line_tpl(line_tpl)
        try:
            year  = int(dpi_obj.dep_date[:4])
        except:
            continue
                
        # sigatm
        if dpi_obj.sigatm == True:
            sigatm_dic = incr_dic(sigatm_dic, year)

        # luzzati
        if dpi_obj.luzatti_free != None:
            luzzati_dic = incr_dic(luzzati_dic, year)
        elif dpi_obj.luzatti_obs != None:
            luzzati_dic = incr_dic(luzzati_dic, year)
        
        # sigma a
        if dpi_obj.sigma_a_free != None:
            sigmaa_dic = incr_dic(sigmaa_dic, year)
        elif dpi_obj.sigma_a_obs != None:
            sigmaa_dic = incr_dic(sigmaa_dic, year)

        # DPI
        if dpi_obj.DPIf != None:
            dpi_dic = incr_dic(dpi_dic, year)
        elif dpi_obj.DPIa != None:
            dpi_dic = incr_dic(dpi_dic, year)
                
        # nr pdbs
        pdb_dic = incr_dic(pdb_dic, year)

    
    fobj.close()
    
    # list of years
    years_lst = pdb_dic.keys() 
    years_lst.sort()
    # totals
    total_dic = {}
    total_dic['sigatm'] = 0
    total_dic['luzzati'] = 0
    total_dic['sigmaa'] = 0
    total_dic['dpi'] = 0
    total_dic['pdb'] = 0
    for year in years_lst:
        if sigatm_dic.has_key(year):
            total_dic['sigatm'] += sigatm_dic[year]
        else:
            sigatm_dic[year] = 0
        if luzzati_dic.has_key(year):
            total_dic['luzzati'] += luzzati_dic[year]
        else:
            luzzati_dic[year] = 0
        if sigmaa_dic.has_key(year):
            total_dic['sigmaa'] += sigmaa_dic[year]
        else:
            sigmaa_dic[year] = 0
        if dpi_dic.has_key(year):
            total_dic['dpi'] += dpi_dic[year]
        else:
            dpi_dic[year] = 0
        total_dic['pdb'] += pdb_dic[year]
            
    # make array
    table_years = Numeric.zeros([5, len(years_lst) + 1], Numeric.Int)
    y_counter = 0
    for y_counter in range(len(years_lst)):
        table_years[0, y_counter] = sigatm_dic[years_lst[y_counter]]
        table_years[1, y_counter] = luzzati_dic[years_lst[y_counter]]
        table_years[2, y_counter] = sigmaa_dic[years_lst[y_counter]]
        table_years[3, y_counter] = dpi_dic[years_lst[y_counter]]
        table_years[4, y_counter] = pdb_dic[years_lst[y_counter]]
        
    table_years[0,len(years_lst)] = total_dic['sigatm']
    table_years[1,len(years_lst)] = total_dic['luzzati']
    table_years[2,len(years_lst)] = total_dic['sigmaa']
    table_years[3,len(years_lst)] = total_dic['dpi']
    table_years[4,len(years_lst)] = total_dic['pdb']
    
    plot_label = years_lst
    plot_label.append('Total')
    print plot_label
    rpy.r.png('pdb_years.png', width = 1200, height = 900)
    rpy.r.barplot(table_years[:,18:], beside = rpy.r.TRUE, names_arg = plot_label[18:], col = ['black', 'blue', 'cyan', 'orange', 'red'], ylim = [0,30000])
    rpy.r.legend(70,29000, legend= ['CoordESD','Luzzati','SigmaA','DPI','PDB'], fill = ['black', 'blue', 'cyan', 'orange', 'red'])    
    rpy.r.dev_off()
    return table_years
    
def incr_dic(x_dic, k):
    if x_dic.has_key(k):
        x_dic[k] += 1
    else:
        x_dic[k] = 1
    return x_dic
    
    
def sus_dist(dpi_file):
    """
    Get distributions of sus: luzzati, sigma A and DPI
    """
    
    fobj = file(dpi_file, 'r')
    lines = fobj.readlines()
    pdb_dic = {}
    for line_str in lines:
        line_str = line_str.strip()
        if line_str[0] == '#':
            continue
        line_tpl = eval(line_str)
        pdb_id = line_tpl[0]
        dpi_obj = sus_dpi.DPI(pdb_id)
        dpi_obj.get_line_tpl(line_tpl)
        pdb_dic[pdb_id] = dpi_obj        
    
    fobj.close()
    
    pdb_lst = pdb_dic.keys()
    dpi_lst = []
    luzatti_lst = []
    sigma_a_lst = []
    # -1 marks not available
    not_defined = -1.0
    for pdb in pdb_lst:
        dpi_obj = pdb_dic[pdb]
        # DPI
        if dpi_obj.DPIf != None:
            dpi_lst.append(dpi_obj.DPIf)
        elif dpi_obj.DPIa != None:
            dpi_lst.append(dpi_obj.DPIa)
        else:
            dpi_lst.append(not_defined)
        # Luzzati
        if dpi_obj.luzatti_free != None:
            luzatti_lst.append(dpi_obj.luzatti_free)
        elif dpi_obj.luzatti_obs != None:
            luzatti_lst.append(dpi_obj.luzatti_obs)
        else:
            luzatti_lst.append(not_defined)
        # sigma a
        if dpi_obj.sigma_a_free != None:
            sigma_a_lst.append(dpi_obj.sigma_a_free)
        elif dpi_obj.sigma_a_obs != None:
            sigma_a_lst.append(dpi_obj.sigma_a_obs)
        else:
            sigma_a_lst.append(not_defined)
        
    dpi_v = Numeric.array(dpi_lst, Numeric.Float)
    defined_dpi_v = Numeric.compress(Numeric.greater_equal(dpi_v, 0), dpi_v)
    
    luzatti_v = Numeric.array(luzatti_lst, Numeric.Float)
    defined_luzatti_v = Numeric.compress(Numeric.greater_equal(luzatti_v,0), luzatti_v)
    
    sigma_a_v = Numeric.array(sigma_a_lst,Numeric.Float)
    defined_sigma_a_v = Numeric.compress(Numeric.greater_equal(sigma_a_v,0), sigma_a_v)
    
    rpy.r.png('box_sus.png')
    rpy.r.boxplot(defined_luzatti_v, defined_sigma_a_v, defined_dpi_v, ylim = [0.0,2.0], names = ['Luzzati', 'SigmaA', 'DPI'], col = ['blue', 'cyan', 'orange'])
    rpy.r.dev_off()
    write_vector('luzzati.sus', defined_luzatti_v)
    write_vector('sigmaA.sus', defined_sigma_a_v)
    write_vector('dpi.sus', defined_dpi_v)
    
    
    
    
    return (defined_dpi_v, defined_luzatti_v, defined_sigma_a_v)

        
                
def write_vector(f_name, v):
    fobj = file(f_name, 'w')
    for c in range(len(v)):
        fobj.write("%f\n" % v[c])
    fobj.close()
    
    
    
def sus_corr(dpi_file):  
    
    fobj = file(dpi_file, 'r')
    lines = fobj.readlines()
    pdb_dic = {}
    for line_str in lines:
        line_str = line_str.strip()
        if line_str[0] == '#':
            continue
        line_tpl = eval(line_str)
        pdb_id = line_tpl[0]
        dpi_obj = sus_dpi.DPI(pdb_id)
        dpi_obj.get_line_tpl(line_tpl)
        pdb_dic[pdb_id] = dpi_obj        
    
    fobj.close()
      
    pdb_lst = pdb_dic.keys()
    dpi_f_lst = []
    dpi_o_lst = []
    luzatti_f_lst = []
    luzatti_o_lst = []
    sigma_a_f_lst = []
    sigma_a_o_lst = []

    # -1 marks not available
    not_defined = -1.0

    for pdb in pdb_lst:
        dpi_obj = pdb_dic[pdb]
        
        if dpi_obj.DPIf != None:
            dpi_f_lst.append(dpi_obj.DPIf)
        else:
            dpi_f_lst.append(-1.0)
        
        if dpi_obj.DPIa != None:
            dpi_o_lst.append(dpi_obj.DPIa)
        else:
            dpi_o_lst.append(-1.0)
            
        if dpi_obj.luzatti_free != None:
            luzatti_f_lst.append(dpi_obj.luzatti_free)
        else:
            luzatti_f_lst.append(-1.0)
            
        if dpi_obj.luzatti_obs != None:
            luzatti_o_lst.append(dpi_obj.luzatti_obs)
        else:
            luzatti_o_lst.append(-1.0)

        if dpi_obj.sigma_a_free != None:
            sigma_a_f_lst.append(dpi_obj.sigma_a_free)
        else:
            sigma_a_f_lst.append(-1.0)
            
        if dpi_obj.sigma_a_obs != None:
            sigma_a_o_lst.append(dpi_obj.sigma_a_obs)
        else:
            sigma_a_o_lst.append(-1.0)
        
    dpi_f = Numeric.array(dpi_f_lst, Numeric.Float)
    dpi_o = Numeric.array(dpi_o_lst, Numeric.Float)
    luzatti_f = Numeric.array(luzatti_f_lst, Numeric.Float)
    luzatti_o = Numeric.array(luzatti_o_lst, Numeric.Float)
    sigma_a_f = Numeric.array(sigma_a_f_lst, Numeric.Float)
    sigma_a_o = Numeric.array(sigma_a_o_lst, Numeric.Float)
        
    dpi_f1 = Numeric.compress(Numeric.greater_equal(dpi_f, 0.0), dpi_f)
    luzatti_f1 = Numeric.compress(Numeric.greater_equal(dpi_f, 0.0), luzatti_f)
    sigma_a_f1 = Numeric.compress(Numeric.greater_equal(dpi_f, 0.0), sigma_a_f)
    dpi_f2 = Numeric.compress(Numeric.greater_equal(luzatti_f1, 0.0), dpi_f1)
    luzatti_f2 = Numeric.compress(Numeric.greater_equal(luzatti_f1, 0.0), luzatti_f1)
    sigma_a_f2 = Numeric.compress(Numeric.greater_equal(luzatti_f1, 0.0), sigma_a_f1)
    dpi_f3 = Numeric.compress(Numeric.greater_equal(sigma_a_f2, 0.0), dpi_f2)
    luzatti_f3 = Numeric.compress(Numeric.greater_equal(sigma_a_f2, 0.0), luzatti_f2)
    sigma_a_f3 = Numeric.compress(Numeric.greater_equal(sigma_a_f2, 0.0), sigma_a_f2)
    
    dpi_f4 = Numeric.compress(Numeric.less(sigma_a_f3, 10.0), dpi_f3)
    luzatti_f4 = Numeric.compress(Numeric.less(sigma_a_f3, 10.0), luzatti_f3)
    sigma_a_f4 = Numeric.compress(Numeric.less(sigma_a_f3, 10.0), sigma_a_f3)

        
    write_vector('dpi_f4.sus',dpi_f4)
    write_vector('luzatti_f4.sus', luzatti_f4)
    write_vector('sigma_a_f4.sus',sigma_a_f4)
    
    return (dpi_f4, luzatti_f4, sigma_a_f4)
        
        
        
def comp_dpi_Rd(dpi_file):
    
    fobj = file(dpi_file, 'r')
    lines = fobj.readlines()
    pdb_dic = {}
    for line_str in lines:
        line_str = line_str.strip()
        if line_str[0] == '#':
            continue
        line_tpl = eval(line_str)
        pdb_id = line_tpl[0]
        dpi_obj = sus_dpi.DPI(pdb_id)
        dpi_obj.get_line_tpl(line_tpl)
        pdb_dic[pdb_id] = dpi_obj        
    
    fobj.close()

    pdb_lst = pdb_dic.keys()
    dpif_lst = []
    dRf_lst = []
    dpia_lst = []
    dRa_lst = []
    for pdb in pdb_lst:
        dpi_obj = pdb_dic[pdb]
        if dpi_obj.DPIf != None:
            dpif_lst.append(dpi_obj.DPIf)
            dRf = dpi_obj.Rfree * dpi_obj.dmin
            dRf_lst.append(dRf)
        if dpi_obj.DPIa != None:
            dpia_lst.append(dpi_obj.DPIa)
            dRa = dpi_obj.R * dpi_obj.dmin
            dRa_lst.append(dRa) 
            
    dpif_v = Numeric.array(dpif_lst, Numeric.Float)
    dRf_v = Numeric.array(dRf_lst, Numeric.Float)
    
    dpia_v = Numeric.array(dpia_lst, Numeric.Float)
    dRa_v =  Numeric.array(dRa_lst, Numeric.Float)
    
    write_vector('comp_dpif.sus',dpif_v)
    write_vector('comp_dRf.sus', dRf_v)
    write_vector('comp_dpia.sus',dpia_v)
    write_vector('comp_dRa.sus', dRa_v)

    

def nobs_dist(dpi_file):
    pdb_dic = {}
    pdb_dic = sus_dpi.read_sus(dpi_file, pdb_dic)
    pdb_lst = pdb_dic.keys()
    nobs_dic = {}
    for pdb in pdb_lst:
        dpi_obj = pdb_dic[pdb]
        if dpi_obj.nobs != None:
            nobs_dic[pdb] = dpi_obj.nobs
    write_dic_as_matrix('nobs.mat', nobs_dic)
    return nobs_dic



def write_dic_as_matrix(f_name, d):
    fobj = file(f_name, 'w')
    for key in d.keys():
        fobj.write("%s %s\n" % (str(key),str(d[key])))
    fobj.close()
    
    
