"""
SCOP database
"""

# Copyright 2007 Max-Planck-Institut Informatik
# 
#     This file is part of STRuster.
# 
#     STRuster is free software; you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation; either version 2 of the License, or
#     (at your option) any later version.
# 
#     STRuster is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
# 
#     You should have received a copy of the GNU General Public License
#     along with STRuster; if not, write to the Free Software
#     Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
#
# Contact:
#     Francisco S. Domingues
#     email: doming@mpi-sb.mpg.de


import cPickle

def get_scop_des(filename):
    """
    Read SCOP descrition file
    return dictionary[sunid] description
    """
    des_dic = {}
    fobj = file(filename, 'r')
    lines = fobj.readlines()
    fobj.close()
    
    for line_str in lines:
        line_str = line_str.strip()
        if line_str[0] == '#' or line_str == '':
            continue
        line_lst = line_str.split(None, 4)
        sunid = line_lst[0]
        des_str = line_lst[4]
        if des_dic.has_key(sunid):
            raise RuntimeError, "Sunid already exists in scop descrition file: %s" % line_str
            continue
        des_dic[sunid] = des_str
    return des_dic


    
def get_scop_cla(filename):
    """
    Read SCOp classification file
    return cla_dic[sid] sentry_dic
    sentry_dic[key]value
        sid , pdb, dom_def, and sunids for cl, cf, sf, fa, dm, sp, px
    """
    fobj = file(filename, 'r')
    lines = fobj.readlines()
    fobj.close()
    
    cla_dic = {}    
    for line_str in lines:
        line_str = line_str.strip()
        if line_str[0] == '#' or line_str == '':
            continue        
        sentry_dic = {}
        line_lst = line_str.split(None)
        sid = line_lst[0]
        if cla_dic.has_key(sid):
            raise RuntimeError, "Sid already exists in scop descrition file: %s" % line_str
        sentry_dic['sid'] =  sid
        sentry_dic['pdb_id'] = line_lst[1]
        sentry_dic['dom_def'] = line_lst[2]
        sunid_lst = line_lst[5].split(',')
        cl_sunid = sunid_lst[0]
        if cl_sunid[:3] != 'cl=':
            raise RuntimeError, "SCOP cla line %s" % line_str
        cl_sunid = cl_sunid[3:]
        sentry_dic['cl'] = cl_sunid
        cf_sunid = sunid_lst[1]
        if cf_sunid[:3] != 'cf=':
            raise RuntimeError, "SCOP cla line %s" % line_str
        cf_sunid = cf_sunid[3:]
        sentry_dic['cf'] = cf_sunid
        sf_sunid = sunid_lst[2]
        if sf_sunid[:3] != 'sf=':
            raise RuntimeError, "SCOP cla line %s" % line_str
        sf_sunid = sf_sunid[3:]
        sentry_dic['sf'] = sf_sunid
        fa_sunid = sunid_lst[3]
        if fa_sunid[:3] != 'fa=':
            raise RuntimeError, "SCOP cla line %s" % line_str
        fa_sunid = fa_sunid[3:]
        sentry_dic['fa'] = fa_sunid
        dm_sunid = sunid_lst[4]
        if dm_sunid[:3] != 'dm=':
            raise RuntimeError, "SCOP cla line %s" % line_str
        dm_sunid = dm_sunid[3:]
        sentry_dic['dm'] = dm_sunid
        sp_sunid = sunid_lst[5]
        if sp_sunid[:3] != 'sp=':
            raise RuntimeError, "SCOP cla line %s" % line_str
        sp_sunid = sp_sunid[3:]
        sentry_dic['sp'] = sp_sunid
        px_sunid = sunid_lst[6]
        if px_sunid[:3] != 'px=':
            raise RuntimeError, "SCOP cla line %s" % line_str
        px_sunid = px_sunid[3:]
        sentry_dic['px'] = px_sunid
        cla_dic[sid] = sentry_dic
    return cla_dic
            
    

def get_struster_des_dic(des_filename, cla_filename, dic_filename):
    des_dic = get_scop_des(des_filename)
    cla_dic = get_scop_cla(cla_filename)
    
    struster_des_dic = {}
    sid_lst = cla_dic.keys()
    sid_lst.sort()
    for sid in sid_lst:
        if struster_des_dic.has_key(sid):
            raise RuntimeError, "Repeating sid %s" % sid        
        sentry_des_dic = {}
        sentry_des_dic['sid'] = sid
        sentry_des_dic['pdb_id'] = cla_dic[sid]['pdb_id']
        sentry_des_dic['dom_def'] = cla_dic[sid]['dom_def']
        sentry_des_dic['sp_sunid'] = cla_dic[sid]['sp']
        sentry_des_dic['cl'] = get_des(sid, 'cl', des_dic, cla_dic)
        sentry_des_dic['cf'] = get_des(sid, 'cf', des_dic, cla_dic) 
        sentry_des_dic['sf'] = get_des(sid, 'sf', des_dic, cla_dic) 
        sentry_des_dic['fa'] = get_des(sid, 'fa', des_dic, cla_dic) 
        sentry_des_dic['dm'] = get_des(sid, 'dm', des_dic, cla_dic) 
        sentry_des_dic['sp'] = get_des(sid, 'sp', des_dic, cla_dic) 
        struster_des_dic[sid] = [sentry_des_dic]
        pdb_id = sentry_des_dic['pdb_id']
        if struster_des_dic.has_key(pdb_id):
            struster_des_dic[pdb_id].append(sentry_des_dic)
        else:
            struster_des_dic[pdb_id] = [sentry_des_dic]
            
    fobj = file(dic_filename, 'w')
    pickle_protocol = 2
    cPickle.dump(struster_des_dic, fobj, pickle_protocol)
    fobj.close()
    print "Wrote: %s" % dic_filename
                   
        
                
def get_des(sid, scop_level, des_dic, cla_dic):
    sunid =  cla_dic[sid][scop_level]
    des = des_dic[sunid]
    return des